<?php

namespace Drupal\tlsrpt;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list controller for the TLSRPT entity type.
 */
class TlsrptListBuilder extends EntityListBuilder {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new TlsrptListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, DateFormatterInterface $date_formatter) {
    parent::__construct($entity_type, $storage);
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build['table'] = parent::render();

    $total = $this->getStorage()
      ->getQuery()
      ->accessCheck(FALSE)
      ->count()
      ->execute();

    $build['summary']['#markup'] = $this->t('Total TLSRPT reports received: @total', ['@total' => $total]);
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    $header = $this->buildHeader();
    return $this->getStorage()
      ->getQuery()
      ->accessCheck(TRUE)
      ->tableSort($header)
      ->pager($this->limit)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['report_id'] = $this->t('Report ID');
    $header['organization_name'] = [
      'data' => $this->t('Organization name'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['total_successful'] = [
      'data' => $this->t('Total successful'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['total_failure'] = [
      'data' => $this->t('Total failures'),
    ];
    $header['received'] = [
      'data' => $this->t('Received'),
      'field' => 'received',
      'specifier' => 'received',
      'sort' => 'desc',
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header += parent::buildHeader();
    $header['operations'] = ['data' => $header['operations']];
    $header['operations']['class'] = [RESPONSIVE_PRIORITY_LOW];
    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var TlsrptInterface $entity */
    $row['report_id']['data'] = [
      '#type' => 'link',
      '#title' => $entity->label(),
      '#url' => $entity->toUrl(),
    ];
    $row['organization_name'] = $entity->get('organization_name')->value;
    $row['total_successful'] = $entity->get('total_successful')->value;
    $row['total_failure'] = $entity->get('total_failure')->value;
    $row['received'] = $this->dateFormatter->format($entity->get('received')->value);
    return $row + parent::buildRow($entity);
  }

}
